@@ -2,10 +2,8 @@
 from __future__ import print_function
 from .__version__ import S_VERSION, S_BUILD_DT
 from .__init__ import EP, PY2
-from . import config as Config
 from . import util as Util
 from . import chat as Chat
-from . import world as World
 from . import diag as Diag
 
 import re
@@ -21,14 +19,6 @@ import gc
 print = Util.print
 
 
-if __name__ == "__main__":
-    raise RuntimeError(
-        "\r\n{0}\r\n\r\n  this file is part of retr0chat.\r\n  enter the parent folder of this file and run:\r\n\r\n    python -m r0c <telnetPort> <netcatPort>\r\n\r\n{0}".format(
-            "*" * 72
-        )
-    )
-
-
 HELP_INTRO = u"""\
 Useful commands:
    \033[36m/nick\033[0m  change your nickname
@@ -38,9 +28,9 @@ Text formatting:
   \033[36mCTRL-O\033[0m  reset text formatting
   \033[36mCTRL-B\033[0m  bold/bright text on/off
   \033[36mCTRL-K\033[0m  followed by a colour code:
-       \033[36m2\033[0m  \033[32mgreen\033[0m,
-    \033[36m15,4\033[0m  \033[1;37;44mbold white on blue\033[0m --
-          say \033[1m/cmap\033[0m to see all options
+       \033[36m2\033[0m  - \033[32mgreen\033[0m,
+    \033[36m15,4\033[0m  - \033[1;37;44mbold white on blue\033[0m --
+            say \033[1m/cmap\033[0m to see all options
 
 Switching channels:
   \033[36mCTRL-E\033[0m  jump to active channel
@@ -49,8 +39,8 @@ Switching channels:
   \033[36m/3\033[0m      go to channel 3
   \033[36m/0\033[0m      go to this channel
 
-Creating or joining the "general" chatroom:
-  \033[36m/join #general\033[0m
+Creating or joining the "g" (general) chatroom:
+  \033[36m/join #g\033[0m
 
 Leaving a chatroom:
   \033[36m/part\033[0m
@@ -65,11 +55,13 @@ Keybinds:
   \033[36mPgUp\033[0m / \033[36mPgDown\033[0m   chatlog scrolling... \033[1mtry it :-)\033[0m
 
 if you are using a mac, PgUp is fn-Shift-PgUp
+if your terminal is tiny, try \033[36m/mn\033[0m and \033[36m/cy\033[0m
 """
 
 
 class User(object):
     def __init__(self, world, address):
+        self.ar = world.ar
         self.world = world
         self.admin = False  # set true after challenge success
         self.client = None  # the client which this object belongs to
@@ -104,7 +96,7 @@ class User(object):
                     yield ch1 + ch2
 
     def set_rand_nick(self):
-        plain_base = Config.NICK_SALT + u"{0}".format(self.client.adr[0])
+        plain_base = self.ar.nsalt + u"{0}".format(self.client.adr[0])
         for suffix in self.pattern_gen():
             plain = plain_base + suffix
             nv = hashlib.sha256(plain.encode("utf-8")).digest()
@@ -133,7 +125,7 @@ class User(object):
                              `34m  b. r0c_build `0m
 """
             # the messy version
-            text = u"""\
+            text = u"""
 `1;30m________ `37m__`36m_ `30m________
 `1;30m░▒▓█▀▀▀▀`37m █▀`46m▓`0;1;30m ▀▀▀▀█▓▒░   `0;36m┌pad1[`0mret`1mr0c`22mhat r0c_ver`36m]pad2┐
 `1;30m ░▒▓ `34;46m▒`0;1;36m▀█ `37;46m▓`0m `1;37;46m▓`0m `1;36m█▀`34m▀ `30m▓▒░    `0;36m│`0mgithub.com/9001/r0c`36m│
@@ -150,7 +142,8 @@ class User(object):
 """
 
             # the messy version
-            text = u"""`1;30m______    `37m_`30m    ______
+            text = u"""
+`1;30m______    `37m_`30m    ______
 `1;30m\\\\\\\\\\\\\\  `37m/ \\  `30m///////   `0mret`1mr0c`22mhat r0c_ver `36m-----
  `1;30m\\\\ `36m/`37m^^  | |  `36m/^`0;36m^`1;30m //    `0mgithub.com/9001/r0c
   `1;30m\\ `0;36m|    `1m\\_/  `0;36m\\__ `1;30m/     `0;36m------b. r0c_build `0m
@@ -175,6 +168,9 @@ class User(object):
         )
         text += HELP_INTRO
 
+        # x = u"`1;30m░▒▓█▀▀▀▀`37m█▀`46m▓`0;1;30m▀▀▀▀█▓▒░`0;36m┌ "
+        # text += x.replace(u"`", u"\033[") * 66
+
         uchan = self.world.join_priv_chan(self, u"r0c-status")
         nchan = uchan.nchan
         nchan.topic = u"r0c readme (and status info)"
@@ -182,38 +178,8 @@ class User(object):
         msg = Chat.Message(nchan, time.time(), u"-nfo-", text)
         nchan.msgs.append(msg)
 
-        if False:
-            text = []
-            lipsum1 = u"Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."
-            lipsum2 = u"Lorem ipsum dolor sit amet, \033[1;31mconsectetur\033[0m adipiscing elit, sed do eiusmod tempor incididunt ut \033[1;32mlabore et dolore magna\033[0m aliqua. Ut enim ad minim veniam, quis nostrud \033[1;33mexercitation ullamco laboris nisi ut aliquip ex ea\033[0m commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est labo\033[1;35mrum."
-            for n in range(10):
-                text.append(lipsum1)
-                text.append(lipsum2)
-            for ln in text:
-                nchan.msgs.append(Chat.Message(nchan, time.time(), u"-nfo-", ln))
-
         self.new_active_chan = uchan
 
-        if False:
-            uchan = self.world.join_pub_chan(self, u"general")
-            nchan = uchan.nchan
-            if len(nchan.msgs) < 100:
-                for n in range(1, 200):
-                    txt = u"{1}_{0:03}     \\\\\\\\".format(
-                        n,
-                        u"_{0:03}     \\\\\\\\\n".format(n).join(
-                            str(v).rjust(v + 4, u" ") for v in range(0, 12)
-                        ),
-                    )
-                    self.world.send_chan_msg(self.nick, nchan, txt)
-
-        if False:
-            uchan = self.world.join_pub_chan(self, u"smalltalk")
-            nchan = uchan.nchan
-            for n in range(1, 3):
-                txt = u"  message {0}\n      mes {0}".format(n)
-                self.world.send_chan_msg(self.nick, nchan, txt)
-
         self.client.handshake_world = True
 
     def admin_test(self, cmd, arg):
@@ -270,7 +236,7 @@ class User(object):
             )
 
         elif cmd == u"auth":
-            if arg == self.world.core.password:
+            if arg == self.ar.pw:
                 self.admin = True
                 self.world.send_chan_msg(u"-nfo-", inf, u"please don't break anything")
             else:
@@ -288,13 +254,16 @@ class User(object):
                 )
                 return
 
-            # TODO: make this more lenient?
             legit_chars = Util.azAZ
-            legit_chars += u"0123456789_-"
+            legit_chars += u"0123456789"
+            legit_within = u"`~!@#$%^&*()_+-=[]{};':\"|,./<>?"
             new_nick = u""
+            err_extra = u""
             for ch in arg:
-                if ch in legit_chars:
+                if ch in legit_chars or (ch in legit_within and new_nick):
                     new_nick += ch
+                elif not new_nick:
+                    err_extra = "  (nicks must begin with an alphanumeric character)\n"
 
             if not new_nick:
                 self.world.send_chan_msg(
@@ -310,17 +279,9 @@ class User(object):
                     u"-err-",
                     inf,
                     u"[invalid argument]\n  "
-                    + u"some illegal characters were removed\n",
+                    + u"some illegal characters were removed\n"
+                    + err_extra,
                 )
-                return
-
-            if new_nick.startswith(u"-"):
-                self.world.send_chan_msg(
-                    u"-err-",
-                    inf,
-                    u"[invalid argument]\n  " + u'nicks cannot start with "-" (dash)\n',
-                )
-                return
 
             if len(new_nick) > 32:
                 self.world.send_chan_msg(
@@ -405,6 +366,9 @@ class User(object):
             )
 
         elif cmd == u"join" or cmd == u"j":
+            if arg and not arg.startswith(u"#"):
+                arg = u"#" + arg
+
             if not arg or len(arg) < 2:
                 self.world.send_chan_msg(
                     u"-err-",
@@ -416,36 +380,11 @@ class User(object):
                 )
                 return
 
-            if not arg.startswith(u"#"):
-                self.world.send_chan_msg(
-                    u"-err-",
-                    inf,
-                    u"""[error]
-  illegal channel name:  {0}
-  channel names must start with #
-""".format(
-                        arg
-                    ),
-                )
-                return
-
             nchan = self.world.join_pub_chan(self, arg[1:]).nchan
             # this is in charge of activating the new channel,
             # rapid part/join will crash us without this
             self.client.refresh(False)
 
-            if False:
-                # measure performance on chans with too many messages
-                if len(self.active_chan.nchan.msgs) < 1048576:
-                    for n in range(0, 1048576):
-                        if n % 16384 == 0:
-                            print(n)
-                        self.world.send_chan_msg(
-                            u"--",
-                            self.active_chan.nchan,
-                            u"large history load test {0}".format(n),
-                        )
-
         elif cmd == u"part" or cmd == u"p":
             if self.active_chan.alias == u"r0c-status":
                 self.world.send_chan_msg(
@@ -453,9 +392,7 @@ class User(object):
                     inf,
                     u"""[error]
   cannot part the status channel
-""".format(
-                        arg
-                    ),
+""",
                 )
                 return
 
@@ -533,6 +470,19 @@ class User(object):
             if not self.admin_test(cmd, arg):
                 return
 
+            try:
+                nt = int(arg1)
+                _ = u"a" in arg2
+            except:
+                self.world.send_chan_msg(
+                    u"-err-",
+                    inf,
+                    u"""[error]
+  bad syntax
+""",
+                )
+                return
+
             for n in range(int(arg1)):
                 self.world.send_chan_msg(
                     self.nick, self.active_chan.nchan, u"{0} {1}".format(arg2, n)
@@ -632,7 +582,12 @@ class User(object):
                 # 	self.new_active_chan.nchan.get_name()))
 
             elif self.old_active_chan:
-                self.new_active_chan = self.old_active_chan
+                ch = self.old_active_chan
+                if ch.nchan in self.world.priv_ch or ch.nchan in self.world.pub_ch:
+                    self.new_active_chan = ch
+                else:
+                    self.new_active_chan = self.chans[0]
+
                 # print('jumping to last active, {0}'.format(
                 # 	self.new_active_chan.nchan.get_name()))
             else:
@@ -706,22 +661,6 @@ class User(object):
 """,
                 )
 
-        elif cmd == u"search" or cmd == u"srch" or cmd == u"s":
-            ch = self.active_chan
-            nch = self.active_chan.nchan
-            if not arg:
-                self.world.send_chan_msg(
-                    u"--",
-                    inf,
-                    u"""[search]
-  plaintext search:
-    /s fore       finds messages like "before"
-
-  regex search:
-    /s s/\\bfore/  finds messages like "foremost"
-""",
-                )
-
         elif cmd == u"sw":
             try:
                 arg = int(arg)
@@ -786,7 +725,7 @@ class User(object):
                     pass
 
             if int_arg is not None:
-                if int_arg > 200:
+                if int_arg > 9000:
                     self.world.send_chan_msg(u"-err-", inf, u"whoa dude")
                     return
 
@@ -831,32 +770,99 @@ class User(object):
             self.world.send_chan_msg(
                 u"--", inf, u"Audible alerts enabled. Disable with /bn", False
             )
+            self.client.save_config()
 
         elif cmd == u"bn":
             self.client.bell = False
             self.world.send_chan_msg(
                 u"--", inf, u"Audible alerts disabled. Enable with /by", False
             )
+            self.client.save_config()
+
+        elif cmd == u"cy":
+            self.client.cnicks = True
+            self.client.need_full_redraw = True
+            self.world.send_chan_msg(
+                u"--", inf, u"Colored nicknames enabled. Disable with /cn", False
+            )
+            self.client.save_config()
+
+        elif cmd == u"cn":
+            self.client.cnicks = False
+            self.client.need_full_redraw = True
+            self.world.send_chan_msg(
+                u"--", inf, u"Colored nicknames disabled. Enable with /cy", False
+            )
+            self.client.save_config()
+
+        elif cmd == u"my":
+            self.client.align = True
+            self.client.need_full_redraw = True
+            self.world.send_chan_msg(
+                u"--", inf, u"Wordwrap margin enabled. Disable with /mn", False
+            )
+            self.client.save_config()
+
+        elif cmd == u"mn":
+            self.client.align = False
+            self.client.need_full_redraw = True
+            self.world.send_chan_msg(
+                u"--", inf, u"Wordwrap margin disabled. Enable with /my", False
+            )
+            self.client.save_config()
+
+        elif cmd == u"sy":
+            try:
+                arg = int(arg)
+            except:
+                arg = 1
+
+            if not self.client.slowmo_tx:
+                self.world.cserial += 1
+
+            self.client.slowmo_tx = arg
+            m = u"Slowmo enabled ({0}). This avoids a bug in telnet.exe on windows, but your memory is probably busted already so you have to reconnect now to fix it. Disable with /sn if you change your mind"
+            self.world.send_chan_msg(u"--", inf, m.format(arg), False)
+            self.client.save_config()
+
+        elif cmd == u"sn":
+            if self.client.slowmo_tx:
+                self.client.slowmo_tx = 0
+                self.world.cserial += 1
+
+            self.world.send_chan_msg(
+                u"--", inf, u"Slowmo disabled. Enable with /sy", False
+            )
+            self.client.save_config()
+
+        elif cmd == u"cfg":
+            m = []
+            keys = "slowmo_tx linemode echo_on vt100 crlf codec bell cnicks align"
+            fmt = u"\033[36m{0}=\033[0m{1!r}" if self.client.vt100 else u"{0}={1!r}"
+            for k in keys.split(" "):
+                m.append(fmt.format(k, getattr(self.client, k)))
+
+            self.world.send_chan_msg(u"--", inf, u" ".join(m), False)
 
         elif cmd == u"cmap":
             msg = u"All foreground colours (0 to f) on default background,\n"
             msg += u"each code wrapped in [brackets] for readability:\n  "
-            for n in range(0, 16):
+            for n in range(16):
                 if n == 8:
                     msg += u"\n  \033[1;3{0}m[{1:x}], ".format(n % 8, n)
                 else:
                     msg += u"\033[3{0}m[{1:x}], ".format(n % 8, n)
 
             msg += u"\033[0m\n\nEach background with black text:\n  \033[30m"
-            for n in range(0, 8):
+            for n in range(8):
                 msg += u"\033[4{0}m 0,{0} ".format(n)
 
             msg += u"\033[0m\n\nEach background with gray text:\n  \033[37m"
-            for n in range(0, 8):
+            for n in range(8):
                 msg += u"\033[4{0}m 7,{0} ".format(n)
 
             msg += u"\033[0m\n\nEach background with white text:\n  \033[1;37m"
-            for n in range(0, 8):
+            for n in range(8):
                 msg += u"\033[4{0}m f,{0} ".format(n)
 
             msg += u"\033[0m\n"
