@@ -1,7 +1,6 @@
 # coding: utf-8
 from __future__ import print_function
 from .__init__ import EP, PY2, INTERP
-from . import config as Config
 from . import util as Util
 from . import chat as Chat
 
@@ -19,28 +18,23 @@ else:
 print = Util.print
 
 
-if __name__ == "__main__":
-    raise RuntimeError(
-        "\r\n{0}\r\n\r\n  this file is part of retr0chat.\r\n  enter the parent folder of this file and run:\r\n\r\n    python -m r0c <telnetPort> <netcatPort>\r\n\r\n{0}".format(
-            "*" * 72
-        )
-    )
-
-
 class World(object):
     def __init__(self, core):
         self.core = core
+        self.ar = core.ar
         self.users = []  # User instances
         self.pub_ch = []  # NChannel instances (public)
         self.priv_ch = []  # NChannel instances (private)
         self.dirty_ch = {}  # Channels that have pending tx
+        self.cntab = {}  # nick color cache
+        self.cserial = 0  # selector configuration serial number
         self.task_queue = Queue()  # Delayed processing of expensive tasks
         self.mutex = threading.RLock()
         self.dirty_flag = threading.Event()  # raise after setting dirty_ch
         Util.py26_threading_event_wait(self.dirty_flag)
 
         # config
-        self.messages_per_log_file = Config.MESSAGES_PER_LOG_FILE
+        self.messages_per_log_file = self.ar.rot_msg
 
         # stats for benchmarking
         self.num_joins = 0
@@ -49,6 +43,14 @@ class World(object):
 
         threading.Thread(target=self.refresh_chans, name="tx_chan").start()
 
+    def shutdown(self):
+        with self.mutex:
+            for chanlist in [self.pub_ch, self.priv_ch]:
+                ucs = [y for x in chanlist for y in x.uchans]
+                ucs = [x for x in ucs if x.alias != u"r0c-status"]
+                for uc in ucs:
+                    self.part_chan(uc, True)
+
     def add_user(self, user):
         with self.mutex:
             self.users.append(user)
@@ -71,7 +73,7 @@ class World(object):
 
             with self.mutex:
                 if not self.dirty_ch:
-                    # raised with no pending work; shutdown signal
+                    # raised with no pending work; maybe shutdown signal
                     continue
 
                 dirty_ch = list(self.dirty_ch)
@@ -102,8 +104,8 @@ class World(object):
                 uchan.user.client.refresh(False)
 
     def send_chan_msg(self, from_nick, nchan, text, ping_self=True):
-        max_hist_mem = Config.MAX_HIST_MEM
-        msg_trunc_size = Config.MSG_TRUNC_SIZE
+        max_hist_mem = self.ar.hist_mem
+        msg_trunc_size = self.ar.hist_tsz
         with self.mutex:
             self.num_messages += 1
             if nchan.name is None and not from_nick.startswith(u"-"):
@@ -170,7 +172,7 @@ class World(object):
                         if isinstance(from_nick, str):
                             Util.whoops("from_nick is bytestring")
 
-                    if uchan.alias == "r0c-status":
+                    if uchan.alias == u"r0c-status":
                         if ping_self:
                             uchan.last_ping = msg.sno
                     else:
@@ -233,7 +235,7 @@ class World(object):
 
     def join_pub_chan(self, user, name):
         with self.mutex:
-            name = name.strip()
+            name = name.strip().lower()
             nchan = self.get_pub_chan(name)
             if nchan is None:
                 nchan = Chat.NChannel(
@@ -249,9 +251,22 @@ class World(object):
                         ),
                     )
                 )
-                # if nchan.name != 'xld':
-                self.load_chat_log(nchan)
-                # self.task_queue.put([self.load_chat_log, [nchan], {}])
+                if nchan.name != "scrolltest":
+                    self.load_chat_log(nchan)
+                    # self.task_queue.put([self.load_chat_log, [nchan], {}])
+                else:
+                    for n1 in range(10):
+                        txt = u""
+                        for n2 in range(10):
+                            txt += u"{0}{1}".format(n1, n2) * 20 + " "
+
+                        msg = Chat.Message(nchan, time.time(), u"--", txt)
+                        nchan.msgs.append(msg)
+
+                    txt = u"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa " * 6
+                    msg = Chat.Message(nchan, time.time(), u"--", txt)
+                    nchan.msgs.append(msg)
+
                 self.pub_ch.append(nchan)
 
             ret = self.join_chan_obj(user, nchan)
@@ -300,7 +315,7 @@ class World(object):
                         )
 
     def broadcast_message(self, msg, severity=1):
-        """ 1=append, 2=append+scroll """
+        """1=append, 2=append+scroll"""
         with self.mutex:
             for nchan in self.pub_ch:
                 self.send_chan_msg(u"--", nchan, msg)
@@ -321,7 +336,7 @@ class World(object):
                             .encode("utf-8")
                         )
 
-    def part_chan(self, uchan):
+    def part_chan(self, uchan, quiet=False):
         with self.mutex:
             self.num_parts += 1
             user = uchan.user
@@ -342,9 +357,10 @@ class World(object):
                     i -= 1
                 user.new_active_chan = user.chans[i]
 
-            self.send_chan_msg(
-                u"--", nchan, u"\033[1;33m{0}\033[22m has left".format(user.nick)
-            )
+            if not quiet:
+                self.send_chan_msg(
+                    u"--", nchan, u"\033[1;33m{0}\033[22m has left".format(user.nick)
+                )
 
             if not nchan.uchans:
                 print(" close chan:  [{0}]".format(nchan.get_name()))
@@ -396,7 +412,7 @@ class World(object):
         ln = u"???"
         t2 = time.time()
         chunks = [nchan.msgs]
-        n_left = Config.MAX_HIST_LOAD - len(nchan.msgs)
+        n_left = self.ar.hist_rd - len(nchan.msgs)
         bytes_loaded = 0
         try:
             for fn in reversed(sorted(files)):
@@ -442,7 +458,7 @@ class World(object):
         t3 = time.time()
         print(
             "  chan hist:  {0} msgs, {1:.0f} kB, {2:.2f} + {3:.2f} sec, #{5}".format(
-                Config.MAX_HIST_LOAD - n_left,
+                self.ar.hist_rd - n_left,
                 bytes_loaded / 1024.0,
                 t2 - t1,
                 t3 - t2,
@@ -459,25 +475,27 @@ class World(object):
 
     def start_logging(self, nchan, chat_backlog=None):
         if nchan.name is not None:
-            log_dir = "{0}chan/{1}".format(EP.log, nchan.name)
+            log_dir = u"{0}chan/{1}".format(EP.log, Util.sanitize_fn(nchan.name))
         else:
-            log_dir = "{0}pm/{1}".format(
-                EP.log, "/".join([x.user.nick for x in nchan.uchans])
+            log_dir = u"{0}pm/{1}".format(
+                EP.log, u"/".join([Util.sanitize_fn(x.user.nick) for x in nchan.uchans])
             )
 
         if nchan.log_fh:
             nchan.log_fh.close()
-        else:
-            try:
-                os.makedirs(log_dir)
-            except:
-                pass
+
+        # always mkdir because direction can turn on reconnect
+        try:
+            os.makedirs(log_dir)
+        except:
+            if not os.path.isdir(log_dir):
+                raise
 
         ts = datetime.utcnow().strftime("%Y-%m%d-%H%M%S")
-        log_fn = "{0}/{1}".format(log_dir, ts)
+        log_fn = u"{0}/{1}".format(log_dir, ts)
 
         while os.path.isfile(log_fn):
-            log_fn += "-"
+            log_fn += u"-"
 
         nchan.log_ctr = 0
         nchan.log_fh = open(log_fn, "wb")
